/******************************************************************
Copyright (c) 1996 Novell, Inc. All Rights Reserved.

THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
TREATIES. USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
THAT CONTAINS THIS WORK. PURSUANT TO THE SDK LICENSE AGREEMENT,
NOVELL HEREBY GRANTS TO DEVELOPER A ROYALTY-FREE, NON-EXCLUSIVE
LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS PRODUCT. NOVELL
GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF
DEVELOPER'S PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO
DEVELOPER OR DEVELOPER'S CUSTOMERS WITH RESPECT TO THIS CODE.
******************************************************************/

package com.novell.nds.dirxml.driver.jdbc.db;


import com.novell.nds.dirxml.driver.jdbc.db.io.Base64Codec;

import java.sql.*;


/**
 *  DB2 JDBC library.
 *  <p>
 *  Library of functions/procedures for DB2 UDB that supports
 *  the shipping IDM JDBC driver preconfigurations.
 *  <p>
 *  This class MUST be installed on the database server for the shipping
 *  preconfigurations to work.  It MUST also be compiled for the 1.1.8 JVM when
 *  deployed on a DB2 UDB 7 server.
 *  <p>
 *  NOTE:  DB2 UDB Version 7 doesn't allow functions to execute any
 *  SQL statements, even if the SQL is read-only.
 */
abstract public
class DB2
{

    //prevents javadoc of default constructor
    private
    DB2() {}


/* --------------------------------------------------------------------------- */

    /* Methods run on the DB2 server and IDM server. */

    /** The length of a default DB2 timestamp (e&#46;g&#46; "<code>2002-01-01-00.00.00.000000</code>") */
    static public  final int LEN_TIMESTAMP      = 26;
    static private final int INDEX_WHITESPACE   = 10;
    static private final int INDEX_HOUR_COLON   = 13;
    static private final int INDEX_MINUTE_COLON = 16;

    static final String STR_WHITESPACE  = " ";
    static final String STR_COLON       = ":";


    static
    String
    ToODBCTimestamp(String db2Timestamp)
    {
        StringBuffer buffer;

        buffer = new StringBuffer(LEN_TIMESTAMP);
        buffer.append(db2Timestamp.substring(0, INDEX_WHITESPACE));
        buffer.append(STR_WHITESPACE);
        buffer.append(db2Timestamp.substring((INDEX_WHITESPACE + 1),
                      INDEX_HOUR_COLON));
        buffer.append(STR_COLON);
        buffer.append(db2Timestamp.substring((INDEX_HOUR_COLON + 1),
                      INDEX_MINUTE_COLON));
        buffer.append(STR_COLON);
        buffer.append(db2Timestamp.substring((INDEX_MINUTE_COLON + 1),
                      LEN_TIMESTAMP));

        return buffer.toString();
    }//ToODBCTimestamp(String)


/* --------------------------------------------------------------------------- */

    /* Methods run exclusively on DB2 server. */


    static private final String CONNECTION_URL = "jdbc:default:connection";
    static private final int LEN_BUFFER = 256;
    static private final int NUM_INDEXES = 3;
    static private final int ERR_DUPLICATE_NAME = -601;
    static private final int ERR_UNDEFINED_NAME = -204;


    /**
     *  Base64-encodes <code>bytes</code>.
     *
     *  @param bytes may be <code>null</code>; may be empty
     *  @return may return <code>null</code>
     */
    public static
    String
    TOBASE64(byte[] bytes)
        throws  SQLException
    {
        String value;

        value = null;
        if(bytes != null)
        {
            value = String.valueOf(Base64Codec.encode(bytes));
        }

        return value;
    }//TOBASE64(byte[])


    /**
     *  Converts a default DB2 timestamp (e&#46;g&#46; "<code>2002-01-01-00.00.00.000000</code>")
     *  to ODBC cannonical format.
     *
     *  @param timestamp may be <code>null</code>
     *  @return may return <code>null</code>
     */
    public static
    String
    TO_ODBC_TIMESTAMP(String timestamp)
    {
        if((timestamp != null) &&
           (timestamp.length() == LEN_TIMESTAMP))
        {
            timestamp = ToODBCTimestamp(timestamp);
        }

        return timestamp;
    }//TO_ODBC_TIMESTAMP(String)


    /**
     *  Stored procedure used by preconfigured driver(s)
     *  to retrieve the last generated primary key for table
     *  INDIRECT.LAST_IDG.
     *
     *  @param O_IDG may be <code>null</code>; may contain <code>null</code>
     */
    public static
    void
    PROC_IDG(long[] O_IDG)
        throws  SQLException
    {
        if(O_IDG != null)
        {
            Statement stmt;
            ResultSet rs;
            Connection connection;

            rs = null;
            stmt = null;
            connection = null;

            try
            {
                long key;
                String sql;

                connection = DriverManager.getConnection(CONNECTION_URL);
                stmt = connection.createStatement();
                sql = "SELECT IDG FROM INDIRECT.LAST_IDG FETCH FIRST 1 ROW ONLY";
                rs = stmt.executeQuery(sql);
                while(rs.next())
                {
                    key = rs.getLong(1);
                    if(rs.wasNull() || key < 0)
                    {
                        //only 43 characters are preserved in the exception
                        throw new SQLException
                            ("Table LAST_IDG contains an illegal value.");
                    }
                    else
                    {
                        O_IDG[0] = key;
                    }
                    break;
                }//while
            }//try
            finally
            {
                DBLib.Close(rs);
                DBLib.Close(stmt);
                DBLib.Close(connection);
            }
        }//if
    }//PROC_IDG()


    /**
     *  Stored procedure used by preconfigured driver(s)
     *  to retrieve the last generated primary key for table
     *  INDIRECT.LAST_IDU.
     *
     *  @param O_IDU may be <code>null</code>; may contain <code>null</code>
     */
    public static
    void
    PROC_IDU(long[] O_IDU)
        throws  SQLException
    {
        if(O_IDU != null)
        {
            Statement stmt;
            ResultSet rs;
            Connection connection;

            rs = null;
            stmt = null;
            connection = null;

            try
            {
                long key;
                String sql;

                connection = DriverManager.getConnection(CONNECTION_URL);
                stmt = connection.createStatement();
                sql = "SELECT IDU FROM INDIRECT.LAST_IDU FETCH FIRST 1 ROW ONLY";
                rs = stmt.executeQuery(sql);
                while(rs.next())
                {
                    key = rs.getLong(1);
                    if(rs.wasNull() || key < 0)
                    {
                        //only 43 characters are preserved in the exception
                        throw new SQLException
                            ("Table LAST_IDU contains an illegal value.");
                    }
                    else
                    {
                        O_IDU[0] = key;
                    }
                    break;
                }//while
            }//try
            finally
            {
                DBLib.Close(rs);
                DBLib.Close(stmt);
                DBLib.Close(connection);
            }
        }//if
    }//PROC_IDU()


    /**
     *  Stored Procedure used by preconfigured driver(s) to move
     *  processed rows from the log table to the processed table.
     *
     *  @param I_SCHEMA the schema name of the log/processed tables;
     *  may be <code>null</code>
     */
    public static
    void
    PROC_CLEANUPLOG(String I_SCHEMA)
        throws  SQLException
    {
        if(DBLib.IsNotValue(I_SCHEMA))
        {
            return;
        }

        Statement stmt;
        ResultSet rs;
        Connection connection;
        boolean auto;

        rs = null;
        stmt = null;
        connection = null;

        try
        {
            StringBuffer buffer;
            String logTableName, historyTableName;
            String indexName;

            I_SCHEMA = I_SCHEMA.toUpperCase();
            buffer = new StringBuffer(LEN_BUFFER);
            buffer.append(I_SCHEMA);
            buffer.append(".");
            buffer.append(I_SCHEMA);
            buffer.append("_PROCESS");
            logTableName = buffer.toString();

            buffer = new StringBuffer(LEN_BUFFER);
            buffer.append(I_SCHEMA);
            buffer.append(".");
            buffer.append(I_SCHEMA);
            buffer.append("_PROCESSED");
            historyTableName = buffer.toString();

            buffer = new StringBuffer(LEN_BUFFER);
            buffer.append(I_SCHEMA);
            buffer.append(".IDX_");
            buffer.append(I_SCHEMA);
            buffer.append("LOG_");
            indexName = buffer.toString();

            connection = DriverManager.getConnection(CONNECTION_URL);

            auto = connection.getAutoCommit();
            if(auto)
            {
                connection.setAutoCommit(false);
            }
                //the same data is SELECTed twice
                connection.setTransactionIsolation(Connection.TRANSACTION_REPEATABLE_READ);

                stmt = connection.createStatement();

                buffer = new StringBuffer(LEN_BUFFER);
                buffer.append("INSERT INTO ");
                buffer.append(historyTableName);
                buffer.append(" SELECT * FROM ");
                buffer.append(logTableName);
                buffer.append(" WHERE STATUS != 'N'");
                stmt.executeUpdate(buffer.toString());

                buffer = new StringBuffer(LEN_BUFFER);
                buffer.append("DELETE FROM ");
                buffer.append(logTableName);
                buffer.append(" WHERE STATUS != 'N'");
                stmt.executeUpdate(buffer.toString());

            connection.commit();

            if(auto)
            {
                connection.setAutoCommit(true);
            }

            for(int i=1; i<=NUM_INDEXES; i++)
            {
                buffer = new StringBuffer(LEN_BUFFER);
                buffer.append("DROP INDEX ");
                buffer.append(indexName);
                buffer.append(i);
                dropObject(stmt, buffer.toString());
            }

            buffer = new StringBuffer(LEN_BUFFER);
            buffer.append("CREATE UNIQUE INDEX ");
            buffer.append(indexName);
            buffer.append("1 ");
            buffer.append("ON ");
            buffer.append(logTableName);
            buffer.append(" (RECORD_ID ASC) ");
            buffer.append("INCLUDE (STATUS ASC) ");
            buffer.append("PCTFREE 10 ");
            buffer.append("MINPCTUSED 10");
            createObject(stmt, buffer.toString());

            buffer = new StringBuffer(LEN_BUFFER);
            buffer.append("CREATE INDEX ");
            buffer.append(indexName);
            buffer.append("2 ");
            buffer.append("ON ");
            buffer.append(logTableName);
            buffer.append(" (EVENT_TIME ASC, RECORD_ID ASC) ");
            buffer.append("PCTFREE 10 ");
            buffer.append("MINPCTUSED 10");
            createObject(stmt, buffer.toString());

            buffer = new StringBuffer(LEN_BUFFER);
            buffer.append("CREATE INDEX ");
            buffer.append(indexName);
            buffer.append("3 ");
            buffer.append("ON ");
            buffer.append(logTableName);
            buffer.append(" (STATUS) ");
            buffer.append("PCTFREE 10 ");
            buffer.append("MINPCTUSED 10");
            createObject(stmt, buffer.toString());
        }//try
        finally
        {
            DBLib.Close(rs);
            DBLib.Close(stmt);
            DBLib.Close(connection);
        }
    }//PROC_CLEANUPLOG()


    private static
    void
    dropObject(Statement stmt, String sql)
        throws SQLException
    {
        //because the drop/create index commands are not atomic, account for the case where the object
        //  in question has already been dropped

        try
        {
            stmt.execute(sql);
        }
        catch(SQLException e)
        {
            if(e.getErrorCode() != ERR_UNDEFINED_NAME)
            {
                throw e;
            }
        }//catch
    }//dropObject(Statement)


    private static
    void
    createObject(Statement stmt, String sql)
        throws SQLException
    {
        //because the drop/create index commands are not atomic, account for the case where the object
        //  in question already exists

        try
        {
            stmt.execute(sql);
        }
        catch(SQLException e)
        {
            if(e.getErrorCode() != ERR_DUPLICATE_NAME)
            {
                throw e;
            }
        }//catch
    }//dropObject(Statement)

}//class DB2
