/******************************************************************
Copyright (c) 1996 Novell, Inc. All Rights Reserved.

THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
TREATIES. USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
THAT CONTAINS THIS WORK. PURSUANT TO THE SDK LICENSE AGREEMENT,
NOVELL HEREBY GRANTS TO DEVELOPER A ROYALTY-FREE, NON-EXCLUSIVE
LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS PRODUCT. NOVELL
GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF
DEVELOPER'S PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO
DEVELOPER OR DEVELOPER'S CUSTOMERS WITH RESPECT TO THIS CODE.
******************************************************************/

package com.novell.nds.dirxml.driver.jdbc.db;


import java.text.ParseException;
import java.util.List;


/**
 * An interface for implementing custom timestamp, date, and time translators.
 * Each implementing class MUST implement a constructor with four
 * <code>java&#46;lang&#46;String</code> arguments.
 * <p>
 * The first argument is the native database timestamp/date/time value.
 * All implementations MUST be able to corectly translate ODBC canoncial
 * formats for timestamp <code>yyyy-mm-dd hh:mm:ss&#46;fffffffff</code>,
 * date <code>yyyy-mm-dd</code> and time <code>hh:mm:ss</code>
 * in addition to native database formats for these types.
 * This is easily accomplished by using the
 * <code>java&#46;sql&#46;Timestamp&#46;valueOf(String)</code>,
 * <code>java&#46;sql&#46;Date&#46;valueOf(String)</code> and
 * <code>java&#46;sql&#46;Time&#46;valueOf(String)</code> methods,
 * respectively.
 * <p>
 * The second argument is the name of the Java class this instance represents
 * (i&#46;e&#46;, "<code>java&#46;sql&#46;Timestamp</code>",
 * "<code>java&#46;sql&#46;Date</code>", or "<code>java&#46;sql&#46;Time</code>"),
 * "<code>java&#46;sql&#46;Timestamp</code>" being the default.  The unqualified
 * class name can also be used (e&#46;g&#46;, "timestamp", "date" or "time"),
 * case-insensitive.
 * <p>
 * The third argument is reserved for future use.
 * <p>
 * The fourth argument is the output format, either "<code>1</code>" (integer) or
 * "<code>2</code>" (string), the default being "<code>1</code>" (integer).
 *
 * @see DBLib#NewTimestamp
 * @see DBLib#NewDate
 * @see DBLib#NewTime
 */
public
interface DBTimeTranslator
{

    /** <code>java&#46;sql&#46;Timestamp</code> */
    public static final String CLASS_TIMESTAMP = java.sql.Timestamp.class.getName();
    /** <code>java&#46;sql&#46;Date</code> */
    public static final String CLASS_DATE      = java.sql.Date.class.getName();
    /** <code>java&#46;sql&#46;Time</code> */
    public static final String CLASS_TIME      = java.sql.Time.class.getName();


    /**
     *  Returns this timestamp/date/time value in the directory canonical
     *  string format (e&#46;g&#46; <code>CCYYMMDDHHMMSSNNNNNNNNN</code>,
     *  <code>CCYYMMDD</code> or <code>HHMMSS</code>) or in the
     *  32-bit integer string format (e&#46;g&#46; "1013368206"),
     *  if translation was successful.
     *
     *  @return an equivalent directory canonical string value
     *  (e&#46;g&#46; "20010101000000000000000", "20010101" or "000000") or a
     *  32-bit integer string value (e&#46;g&#46; "1013368206") if conversion
     *  was successful; a native database timestamp otherwise; never
     *  returns <code>null</code>
     */
    public
    String
    getDSTime();


    /**
     *  Translates a database-native timestamp/date/time value into an
     *  equivalent directory format as per the arguments passed to the constructor.
     *
     *  @param nativeTime a database-native timestamp, date, or time value;
     *  may be <code>null</code>
     */
    public
    void
    setDSTime(String nativeTime)
        throws  ParseException,
                IllegalArgumentException;


    /**
     * Indicates whether an error occurred parsing this timestamp/date/time
     * value.
     *
     * @return <code>true</code> if there was a parse error;
     * <code>false</code> otherwise
     */
    public
    boolean
    parseError();


    /**
     * Returns the first parse error message, if any.
     *
     * @return a string with length > 0 if there was a parse error;
     * empty string otherwise; never returns <code>null</code>
     */
    public
    String
    getErrorMessage();


    /**
     * Returns any exceptions generated while parsing.  Returns a List of
     * <code>Exception</code>s with length > 0 if exceptions were generated;
     * <code>Collections&#46;EMPTY_LIST</code> otherwise.
     *
     * @return <code>true</code> if there was a parse error;
     * <code>false</code> otherwise; never returns <code>null</code>
     */
    public
    List
    getExceptions();


    /**
     * Indicates whether precision was lost translating this timestamp
     * value to the specified directory format.
     *
     * @return <code>true</code> if precision was lost;
     * <code>false</code> otherwise
     */
    public
    boolean
    precisionLost();

}//interface DBTimeTranslator
